/* -*- mode: c++ -*- */
#ifndef FR_PROC_DATA_I
#define FR_PROC_DATA_I

%module frameCPP

%{
#include "general/types.hh"
#include "framecpp/FrProcData.hh"
%}

%import "framecpp/FrameCPP.hh"
%import "framecpp/FrProcData.hh"

%import "Container.i"

%include "general/SharedPtr.i"
%SharedPtr(FrProcData)

#define type_type_		INT_2U
#define subType_type_		INT_2U
#define timeOffset_type_	REAL_8
#define tRange_type_		REAL_8
#define fShift_type_		REAL_8
#define phase_type_		REAL_4
#define fRange_type_		REAL_8
#define BW_type_		REAL_8

class FrProcData
{
public:
#define DECLARE(X) \
  typedef X##_ X

  DECLARE(timeOffset_type);
  DECLARE(tRange_type);
  DECLARE(fShift_type);
  DECLARE(phase_type);
  DECLARE(fRange_type);
  DECLARE(BW_type);

#undef DECLARE

#ifdef SWIGPYTHON
  typedef Container< FrVect > data_type;
#else
  using FrameCPP::FrProcData::data_type;
#endif

  enum type_type {
    UNKNOWN_TYPE = FrameCPP::FrProcData::UNKNOWN_TYPE,
    TIME_SERIES = FrameCPP::FrProcData::TIME_SERIES,
    FREQUENCY_SERIES = FrameCPP::FrProcData::FREQUENCY_SERIES,
    OTHER_1D_SERIES_DATA = FrameCPP::FrProcData::OTHER_1D_SERIES_DATA,
    TIME_FREQUENCY = FrameCPP::FrProcData::TIME_FREQUENCY,
    WAVELETS = FrameCPP::FrProcData::WAVELETS,
    MULTI_DIMENSIONAL = FrameCPP::FrProcData::MULTI_DIMENSIONAL
  };

  enum subType_type {
    UNKNOWN_SUB_TYPE = FrameCPP::FrProcData::UNKNOWN_SUB_TYPE,
    //---------------------------------------------------------------
    // Subtype for fSeries
    //---------------------------------------------------------------
    DFT = FrameCPP::FrProcData::DFT,
    AMPLITUDE_SPECTRAL_DENSITY = FrameCPP::FrProcData::AMPLITUDE_SPECTRAL_DENSITY,
    POWER_SPECTRAL_DENSITY = FrameCPP::FrProcData::POWER_SPECTRAL_DENSITY,
    CROSS_SPECTRAL_DENSITY = FrameCPP::FrProcData::CROSS_SPECTRAL_DENSITY,
    COHERENCE = FrameCPP::FrProcData::COHERENCE,
    TRANSFER_FUNCTION = FrameCPP::FrProcData::TRANSFER_FUNCTION
  };

  FrProcData( );
  FrProcData( const std::string& Name,
	      const std::string& Comment,
	      type_type Type,
	      subType_type SubType,
	      timeOffset_type TimeOffset,
	      tRange_type TRange,
	      fShift_type FShift,
	      phase_type Phase,
	      fRange_type FRange,
	      BW_type BW );
  const std::string& GetName() const; 
  const std::string& GetComment( ) const;
  type_type GetType( ) const;
  subType_type GetSubType( ) const;
  timeOffset_type GetTimeOffset( ) const;
  tRange_type GetTRange( ) const;
  fShift_type GetFShift( ) const;
  phase_type GetPhase( ) const;
  fRange_type GetFRange( ) const;
  BW_type GetBW( ) const;

  %extend {
    void
    AppendData( FrVect& Data )
    {
      self->RefData( ).append( Data );
    }

    //-------------------------------------------------------------------
    //-------------------------------------------------------------------
    size_t RefDataSize( );
    //-------------------------------------------------------------------
    //-------------------------------------------------------------------
    data_type& RefData( );
  }	
  %insert("python") %{
    name = property( fget=lambda d:d.GetName( ) )
    comment = property( fget=lambda d:d.GetComment( ) )
    type = property( fget=lambda d:d.GetType( ) )
    subType = property( fget=lambda d:d.GetSubType( ) )
    timeOffset = property( fget=lambda d:d.GetTimeOffset( ) )
    tRange = property( fget=lambda d:d.GetTRange( ) )
    fShift = property( fget=lambda d:d.GetFShift( ) )
    phase = property( fget=lambda d:d.GetPhase( ) )
    fRange = property( fget=lambda d:d.GetFRange( ) )
    BW = property( fget=lambda d:d.GetBW( ) )
    data = property( fget=lambda d:d.RefData( ) )
  %}
};

CONTAINER_WRAP(FrProcData,FrProcData)

%{
  FrProcData::data_type&
  FrProcData_RefData( FrProcData* Proc )
  {
    return Proc->RefData( );
  }

  size_t
  FrProcData_RefDataSize( FrProcData* Proc )
  {
    return Proc->RefData( ).size( );
  }
%}

%init {
#undef VALIDATE
#define VALIDATE(MEMBER)	\
  VALIDATE_DATA_TYPE(FrameCPP::FrProcData,MEMBER##_type,MEMBER##_type_)

  VALIDATE(type);
  VALIDATE(subType);
  VALIDATE(timeOffset);
  VALIDATE(tRange);
  VALIDATE(fShift);
  VALIDATE(phase);
  VALIDATE(fRange);
  VALIDATE(BW);

#undef VALIDATE
}

#undef TYPE_TYPE

#endif /* FR_PROC_DATA_I */
