#ifndef FrameCPP_VERSION_8_FrAdcData_HH
#define FrameCPP_VERSION_8_FrAdcData_HH

#if ! defined(SWIG)
#include <memory>
#include <vector>

#include "general/types.hh"

#include "framecpp/Common/Container.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version7/FrAdcData.hh"

#include "framecpp/Version8/FrameSpec.hh"
#include "framecpp/Version8/Dimension.hh"
#include "framecpp/Version8/FrVect.hh"
#include "framecpp/Version8/STRING.hh"
#endif /* ! defined(SWIG) */

#undef FR_ADC_DATA_CHANNELGROUP_TYPE
#undef FR_ADC_DATA_CHANNELNUMBER_TYPE
#undef FR_ADC_DATA_NBITS_TYPE
#undef FR_ADC_DATA_BIAS_TYPE
#undef FR_ADC_DATA_SLOPE_TYPE
#undef FR_ADC_DATA_SAMPLERATE_TYPE
#undef FR_ADC_DATA_TIMEOFFSET_TYPE
#undef FR_ADC_DATA_FSHIFT_TYPE
#undef FR_ADC_DATA_PHASE_TYPE
#undef FR_ADC_DATA_DATAVALID_TYPE

#define FR_ADC_DATA_CHANNELGROUP_TYPE	INT_4U
#define FR_ADC_DATA_CHANNELNUMBER_TYPE	INT_4U
#define FR_ADC_DATA_NBITS_TYPE		INT_4U
#define FR_ADC_DATA_BIAS_TYPE		REAL_4
#define FR_ADC_DATA_SLOPE_TYPE		REAL_4
#define FR_ADC_DATA_SAMPLERATE_TYPE	REAL_8
#define FR_ADC_DATA_TIMEOFFSET_TYPE	REAL_8
#define FR_ADC_DATA_FSHIFT_TYPE		REAL_8
#define FR_ADC_DATA_PHASE_TYPE		REAL_4
#define FR_ADC_DATA_DATAVALID_TYPE	INT_2U

namespace FrameCPP
{
  namespace Version_8
  {
    const FR_ADC_DATA_BIAS_TYPE FR_ADC_DATA_DEFAULT_BIAS = 0.0;
    const FR_ADC_DATA_DATAVALID_TYPE FR_ADC_DATA_DEFAULT_DATA_VALID = 0;
    const FR_ADC_DATA_FSHIFT_TYPE FR_ADC_DATA_DEFAULT_FSHIFT = 0.0;
    const FR_ADC_DATA_PHASE_TYPE FR_ADC_DATA_DEFAULT_PHASE = 0.0;
    const FR_ADC_DATA_SLOPE_TYPE FR_ADC_DATA_DEFAULT_SLOPE = 1.0;
    const FR_ADC_DATA_TIMEOFFSET_TYPE FR_ADC_DATA_DEFAULT_TIME_OFFSET = 0.0;
    inline const char* FR_ADC_DATA_DEFAULT_UNITS( )
    {
      return "counts";
    }

    //===================================================================
    /// \brief ADC Data
    //===================================================================
    class FrAdcData
      : public ObjectWithChecksum< Common::FrameSpec::Object >,
	public Common::TOCInfo
    {
    public:
      typedef std::string			name_type;
      typedef std::string			comment_type;
      typedef FR_ADC_DATA_CHANNELGROUP_TYPE	channelGroup_type;
      typedef FR_ADC_DATA_CHANNELNUMBER_TYPE	channelNumber_type;
      typedef FR_ADC_DATA_NBITS_TYPE		nBits_type;
      typedef FR_ADC_DATA_BIAS_TYPE		bias_type;
      typedef FR_ADC_DATA_SLOPE_TYPE		slope_type;
      typedef std::string			units_type;
      typedef FR_ADC_DATA_SAMPLERATE_TYPE	sampleRate_type;
      typedef FR_ADC_DATA_TIMEOFFSET_TYPE	timeOffset_type;
      typedef FR_ADC_DATA_FSHIFT_TYPE		fShift_type;
      typedef FR_ADC_DATA_PHASE_TYPE		phase_type;
      typedef FR_ADC_DATA_DATAVALID_TYPE	dataValid_type;

      typedef FrVect					data_value_type;
      typedef Common::Container< data_value_type >	data_type;
      typedef data_type::const_iterator const_iterator;
      typedef data_type::iterator iterator;

      typedef Common::Container< FrVect >	aux_type;
      typedef aux_type::const_iterator const_aux_iterator;
      typedef aux_type::iterator aux_iterator;

      typedef General::SharedPtr< FrAdcData >	subset_ret_type;

      static const REAL_4 DEFAULT_BIAS;
      static const REAL_4 DEFAULT_SLOPE;
      static const REAL_8 DEFAULT_FSHIFT;
      static const REAL_8 DEFAULT_TIME_OFFSET;
      static const INT_2U DEFAULT_DATA_VALID;
      static const REAL_4 DEFAULT_PHASE;

      inline static const char* DEFAULT_UNITS( )
      {
	return FR_ADC_DATA_DEFAULT_UNITS( );
      }

      //-----------------------------------------------------------------
      /// \brief Default constructor
      ///
      /// \return
      ///    A new instance of this object.
      //-----------------------------------------------------------------
      FrAdcData( );

      //-----------------------------------------------------------------
      /// \brief Copy Constructor
      ///
      /// \param[in] Source
      ///     The object from which to copy the information.
      ///
      /// \return
      ///    A new instance of this object.
      //-----------------------------------------------------------------
      FrAdcData( const FrAdcData& Source );

      //-----------------------------------------------------------------
      /// \brief Constructor
      ///
      /// \param[in] name
      ///     The Channel name -- must be unique within the frame
      /// \param[in] group
      ///     Channel grouping number containing ADC.
      /// \param[in] nBits
      ///     Number of bits in A/D output.
      /// \param[in] bias
      ///     DC bias on channel (Units @ADC counts = 0)
      /// \param[in] slope
      ///     ADC calibration: input units/count.
      /// \param[in] units
      ///     ADC calibration: input units for slope.
      /// \param[in] fShift
      ///     The frequency (in Hz) in the origional data that
      ///     oorresponds to 0 Hz in the heterodyned series.
      /// \param[in] timeOffset
      ///     Offset of the first sample relative to the frame start
      ///     time (seconds). Must be positive and smaller than the
      ///     frame length.
      /// \param[in] dataValid
      ///     Data valid flag.
      /// \param[in] phase
      ///     Phase of heterodyning signal at start of dataset.
      ///
      /// \return
      ///     A new instance of the object.
      //-----------------------------------------------------------------
      FrAdcData( const name_type& name, INT_4U group,
		 channelGroup_type channel,
		 nBits_type nBits,
		 sampleRate_type sampleRate,
		 bias_type bias = DEFAULT_BIAS,
		 slope_type slope = DEFAULT_SLOPE,
		 const units_type& units = DEFAULT_UNITS( ),
		 fShift_type fShift = DEFAULT_FSHIFT,
		 timeOffset_type timeOffset = DEFAULT_TIME_OFFSET,
		 dataValid_type dataValid = DEFAULT_DATA_VALID,
		 phase_type phase = DEFAULT_PHASE );

      //-----------------------------------------------------------------
      /// \brief Promotion Constructor
      ///
      /// \param[in] Source
      ///     An FrAdcData structure from a previous frame specification.
      /// \param[in] Stream
      ///     The stream from which the earliest version of the
      ///     FrAdcData structure was read.
      ///
      /// \return
      ///     A new instance of the object.
      //-----------------------------------------------------------------
      FrAdcData( Previous::FrAdcData& Source, Common::IStream* Stream );

      //-----------------------------------------------------------------
      /// \brief Destructor
      //-----------------------------------------------------------------
      ~FrAdcData( );

      //-----------------------------------------------------------------
      /// \brief The name structure name of this object.
      ///
      /// \return
      ///     The name of the structure as specified by the frame
      ///     specification.
      //-----------------------------------------------------------------
      static const char* const StructName( );

      //-----------------------------------------------------------------
      /// \brief The description of structure
      ///
      /// \return
      ///     A Description object which describes this structure as
      ///     specified by the frame specification.
      //-----------------------------------------------------------------
      static const Common::Description* StructDescription( );

      //-----------------------------------------------------------------
      /// \brief equality operator
      ///
      /// \param[in] RHS
      ///     The FrAdcData object to be compared.
      ///
      /// \return
      ///     The value true is returned if this object is equivelent
      ///     to the RHS instance; false otherwise.
      //-----------------------------------------------------------------
      bool operator==( const FrAdcData& RHS ) const;

      //-----------------------------------------------------------------
      /// \brief inequality operator
      ///
      /// \param[in] RHS
      ///     The FrAdcData object to be compared.
      ///
      /// \return
      ///     The value true is returned if this object is not equivelent
      ///     to the RHS instance; false otherwise.
      //-----------------------------------------------------------------
      bool operator!=( const FrAdcData& RHS ) const;

      //-----------------------------------------------------------------
      /// \brief equality operator for abstract data type
      ///
      /// \param[in] RHS
      ///     The object to be compared.
      ///
      /// \return
      ///     The value true is returned if this object is equivelent
      ///     to the RHS instance; false otherwise.
      //-----------------------------------------------------------------
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

      //-----------------------------------------------------------------
      /// \brief The name structure name of this object.
      ///
      /// \return
      ///     The name of the structure as specified by the frame
      ///     specification.
      //-----------------------------------------------------------------
      virtual const char* const ObjectStructName( ) const;

      //-----------------------------------------------------------------
      /// \brief Return the name associate with the FrAdcData structure.
      ///
      /// \return
      ///     The name associated with the FrAdcData structure
      //-----------------------------------------------------------------
      const name_type& GetName() const; 

      //-----------------------------------------------------------------
      /// \brief A brief description of the FrAdcData structure
      ///
      /// \return
      ///     A brief description of the FrAdcData structure
      //-----------------------------------------------------------------
      const comment_type& GetComment( ) const;

      //-----------------------------------------------------------------
      /// \brief The channel number for the FrAdcData
      ///
      /// \return
      ///     The channel number for the FrAdcData
      //-----------------------------------------------------------------
      channelNumber_type GetChannelNumber( ) const;

      //-----------------------------------------------------------------
      /// \brief The channel group for the FrAdcData
      ///
      /// \return
      ///     The channel group for the FrAdcData
      //-----------------------------------------------------------------
      channelGroup_type GetChannelGroup( ) const;

      //-----------------------------------------------------------------
      /// \brief Tne number of bits in the A/D output.
      ///
      /// \return
      ///      Tne number of bits in the A/D output.
      //-----------------------------------------------------------------
      nBits_type GetNBits( ) const;

      //-----------------------------------------------------------------
      /// \brief DC bias on channel.
      ///
      /// \return
      ///     DC bias on channel.
      //-----------------------------------------------------------------
      bias_type GetBias( ) const;

      //-----------------------------------------------------------------
      /// \brief ADC calibration units per count.
      ///
      /// \return
      ///     ADC calibration units per count.
      //-----------------------------------------------------------------
      slope_type GetSlope( ) const;

      //-----------------------------------------------------------------
      /// \brief The units used in the ADC calibration
      ///
      /// \return
      ///     The units used in the ADC calibration
      //-----------------------------------------------------------------
      STRING GetUnits( ) const;

      //-----------------------------------------------------------------
      /// \brief The data acquisition rate.
      ///
      /// \return
      ///     The data acquisition rate.
      //-----------------------------------------------------------------
      sampleRate_type GetSampleRate( ) const;

      //-----------------------------------------------------------------
      /// \brief The offset of the first sample relative to the frame start time.
      ///
      /// \return
      ///     The offset of the first sample relative to the frame
      ///     start time.
      //-----------------------------------------------------------------
      timeOffset_type GetTimeOffset( ) const;

      //-----------------------------------------------------------------
      /// \brief The frequency in the original data corresponding to 0 Hz.
      ///
      /// \return
      ///     The frequency in the original data corresponding to 0 Hz.
      //-----------------------------------------------------------------
      fShift_type GetFShift( ) const;
      
      //-----------------------------------------------------------------
      /// \brief The phase of hetrodyning signal at start of the dataset.
      ///
      /// \return
      ///     The phase of hetrodyning signal at start of the dataset.
      //-----------------------------------------------------------------
      phase_type GetPhase( ) const;

      //-----------------------------------------------------------------
      /// \brief Retrieve the dataValid value.
      ///
      /// \return
      ///     The value of the dataValid field.
      //-----------------------------------------------------------------
      dataValid_type GetDataValid( ) const;

      //-----------------------------------------------------------------
      /// \brief Identifier for vector for user-provided information.
      ///
      /// \return
      ///     A constant reference to the user-provided information.
      //-----------------------------------------------------------------
      const aux_type& RefAux( ) const;

      //-----------------------------------------------------------------
      /// \brief Identifier for vector for user-provided information.
      ///
      /// \return
      ///     A reference to the user-provided information.
      //-----------------------------------------------------------------
      aux_type& RefAux( );

      //-----------------------------------------------------------------
      /// \brief Identifier for vector of sampled data.
      ///
      /// \return
      ///     A constant reference to the sampled data.
      //-----------------------------------------------------------------
      const data_type& RefData( ) const;

      //-----------------------------------------------------------------
      /// \brief Identifier for vector of sampled data.
      ///
      /// \return
      ///     A reference to the sampled data.
      //-----------------------------------------------------------------
      data_type& RefData( );
   
      //-----------------------------------------------------------------
      /// \brief Extend the comment associated with the FrAdcData structure.
      //-----------------------------------------------------------------
      void AppendComment( const comment_type& comment );

      //-----------------------------------------------------------------
      /// \brief Merge with another FrAdcData
      ///
      /// \param[in] RHS
      ///     The source of the information to append to this FrAdcData
      ///     structure.
      ///
      /// \return
      ///     A reference to this object
      //-----------------------------------------------------------------
      FrAdcData& Merge( const FrAdcData& RHS );

      //-----------------------------------------------------------------
      /// \brief Promote previous version of object to this version
      ///
      /// \param[in] Source
      ///     The frame specification version of the object.
      /// \param[in] Obj
      ///     The previous version of the object.
      /// \param[in] Stream
      ///     Stream from which the origional object was created.
      ///
      /// \return
      ///     A new instance of FrameH based on Obj
      //-----------------------------------------------------------------
      static promote_ret_type
      Promote( INT_2U Source,
	       promote_arg_type Obj,
	       Common::IStream* Stream );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the dataValid field.
      ///
      /// \param[in] flag
      ///     The desired value of the dataValid field.
      //-----------------------------------------------------------------
      void SetDataValid( dataValid_type flag );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the bias field.
      ///
      /// \param[in] bias
      ///     The desired value of the bias field.
      //-----------------------------------------------------------------
      void SetBias( bias_type bias );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the sampleRate field.
      ///
      /// \param[in] SampleRate
      ///     The desired value of the sampleRate field.
      //-----------------------------------------------------------------
      void SetSampleRate( sampleRate_type SampleRate );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the slope field.
      ///
      /// \param[in] slope
      ///     The desired value of the slope field.
      //-----------------------------------------------------------------
      void SetSlope( slope_type slope );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the timeOffset field.
      ///
      /// \param[in] time
      ///     The desired value of the timeOffset field.
      //-----------------------------------------------------------------
      void SetTimeOffset( timeOffset_type time );

      //-----------------------------------------------------------------
      /// \brief Request a subset of the data.
      ///
      /// \param[in] Offset
      ///     The number of seconds from the start of the FrAdcData
      ///     object from which to start extracting data.
      /// \param[in] Dt
      ///     The number of seconds of data in the resulting FrAdcData
      ///     strucutre.
      ///
      /// \return
      ///     A FrAdcData structure which contains Dt seconds of data
      ///     starting at Offset of this FrAdcData structure.
      //-----------------------------------------------------------------
      subset_ret_type Subset( REAL_8 Offset, REAL_8 Dt ) const;
   
      //-----------------------------------------------------------------
      /// \brief Answer requests for information as needed by TOC
      ///
      /// \param[in] InfoClass
      ///     Class of information being requested.
      //-----------------------------------------------------------------
      virtual void TOCQuery( int InfoClass, ... ) const;

    protected:
      //-----------------------------------------------------------------
      /// \brief Down grade an object
      ///
      /// \param[in] Target
      ///     The version of the frame specification to demote too.
      /// \param[in] Stream
      ///     The input stream from which the original object was read.
      ///
      /// \return
      ///     An object of the previous generation.
      //-----------------------------------------------------------------
      virtual demote_ret_type
      demote( INT_2U Target,
	      demote_arg_type Obj,
	      Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Upgrade an object
      ///
      /// \param[in] Target
      ///     The version of the frame specification to promote too.
      /// \param[in] Stream
      ///     The input stream from which the original object was read.
      ///
      /// \return
      ///     An object of this class.
      //-----------------------------------------------------------------
      virtual promote_ret_type
      promote( INT_2U Target,
	       promote_arg_type Obj,
	       Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      ///
      /// \param[in] Stream
      ///     The stream from which to the object is being read or
      ///     written.
      ///
      /// \return
      ///     The number of bytes need to read or write this object.
      //-----------------------------------------------------------------
      cmn_streamsize_type pBytes( const Common::StreamBase& Stream) const;

      //-----------------------------------------------------------------
      /// \brief Virtual constructor
      ///
      /// \param[in] Stream
      ///     The input stream from where the object is being read.
      //-----------------------------------------------------------------
      virtual FrAdcData* pCreate( Common::IStream& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Write the structure to the stream
      ///
      /// \param[in] Stream
      ///     The output stream where the object is to be written.
      //-----------------------------------------------------------------
      virtual void pWrite( Common::OStream& Stream ) const;

    private:
      //-----------------------------------------------------------------
      /// \brief Object with checksum data.
      //-----------------------------------------------------------------
      typedef ObjectWithChecksum< Common::FrameSpec::Object > object_type;

      using Common::FrameSpec::Object::Create;

      struct data_definition_type {
	//---------------------------------------------------------------
	/// Channel name  -- must be unique with the frame
	//---------------------------------------------------------------
	STRING	name;
	//---------------------------------------------------------------
	/// Comment for the ADC.
	//---------------------------------------------------------------
	STRING	comment;
	//---------------------------------------------------------------
	/// Channel grouping number containing ADC.
	///
	/// \note
	///     This variable is determined by the site and must be
	///     unique over all detectors.
	//---------------------------------------------------------------
	channelGroup_type	channelGroup;
	//---------------------------------------------------------------
	/// Channel number
	///
	/// \note
	///     This variable is determined by the site and must be
	///     unique over all detectors.
	//---------------------------------------------------------------
	channelNumber_type	channelNumber;
	//---------------------------------------------------------------
	/// Number of bits in A/D output.
	//---------------------------------------------------------------
	nBits_type	nBits;
	//---------------------------------------------------------------
	/// DC bias on channel (Units @ ADC counts = 0)
	//---------------------------------------------------------------
	bias_type	bias;
	//---------------------------------------------------------------
	/// ADC calibration: input units/count.
	//---------------------------------------------------------------
	slope_type	slope;
	//---------------------------------------------------------------
	/// ADC calibration: input units for slope
	/// If dimensionless, then units == <NONE>, in CAPITALS
	/// (without <...>).
	//---------------------------------------------------------------
	STRING	units;
	//---------------------------------------------------------------
	/// Data acquisition rate, samples per second.
	//---------------------------------------------------------------
	sampleRate_type	sampleRate;
	//---------------------------------------------------------------
	/// Offset of first sample relative to the frame start time
	/// in seconds.
	/// The value must be positive and smaller than the frame length.
	///
	/// \note
	///     Time offsets always added together (to obtain decimal
	///     time offset: time of the first sample equals
	///     GTime + timeOffset.
	///     All quantities are ALWAYS positive.
	//---------------------------------------------------------------
	timeOffset_type	timeOffset;
	//---------------------------------------------------------------
	/// The frequency (in HZ) in the original data that corresponds
	/// to zero Hz in the heterodyned series.
	///
	/// \note
	///     In the heterodyning process the real time series in
	///     multiplied by cos[2pi fShift(t-to) + phase] to get
	///     the real part and by -sin[2pi fShift(t-t(0)) + phase] to
	///     get the imaginary part of the resulting complex time
	///     series.
	///     The time origin t(0) is the beginning of the frame.
	//---------------------------------------------------------------
	fShift_type	fShift;
	//---------------------------------------------------------------
	/// Phase (in radian) of heterodyning signal at start of dataset.
	///
	/// \note
	///     In the heterodyning process the real time series in
	///     multiplied by cos[2pi fShift(t-to) + phase] to get
	///     the real part and by -sin[2pi fShift(t-t(0)) + phase] to
	///     get the imaginary part of the resulting complex time
	///     series.
	///     The time origin t(0) is the beginning of the frame.
	//---------------------------------------------------------------
	phase_type	phase;
	//---------------------------------------------------------------
	/// Data valid flag: If the value is zero, the ADC data is valid.
	/// If the value is non-zero, the ADC data should be suspect
	/// or not valid.
	//---------------------------------------------------------------
	dataValid_type	dataValid;
	//---------------------------------------------------------------
	/// Identifier for vector of sampled data.
	//---------------------------------------------------------------
	data_type	data;
	//---------------------------------------------------------------
	/// Identifier fro vector for user-provided information.
	/// Its use is generic.
	//---------------------------------------------------------------
	aux_type	aux;

	//---------------------------------------------------------------
	//---------------------------------------------------------------
	data_definition_type( );

	//---------------------------------------------------------------
	//---------------------------------------------------------------
	data_definition_type( const data_definition_type& Source );

	//---------------------------------------------------------------
	/// \brief Comparison operator
	//---------------------------------------------------------------
	bool operator==( const data_definition_type& RHS ) const;

	//---------------------------------------------------------------
	//---------------------------------------------------------------
	bool operator!=( const data_definition_type& RHS ) const;

	//---------------------------------------------------------------
	//---------------------------------------------------------------
	void copy_core( const data_definition_type& Source );
      } m_data;

      //-----------------------------------------------------------------
      /// \brief Stream Constructor
      ///
      /// \param[in] Stream
      ///     The stream from which the object is being read.
      ///
      /// \return
      ///    A new instance of this object.
      //-----------------------------------------------------------------
      FrAdcData( Common::IStream& Stream );
    };

    inline FrAdcData::promote_ret_type FrAdcData::
    Promote( INT_2U Source,
	     promote_arg_type Obj,
	     Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrAdcData, FrAdcData >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* const FrAdcData::
    StructName( )
    {
      static const char* const name = "FrAdcData";
      return name;
    }

    inline const FrAdcData::comment_type& FrAdcData::
    GetComment( ) const
    {
      return m_data.comment;
    }

    inline FrAdcData::channelNumber_type FrAdcData::
    GetChannelNumber( ) const
    { 
      return m_data.channelNumber;
    }

    inline FrAdcData::channelGroup_type FrAdcData::
    GetChannelGroup( ) const
    {
      return m_data.channelGroup;
    }

    inline FrAdcData::nBits_type FrAdcData::
    GetNBits( ) const
    {
      return m_data.nBits;
    }

    inline FrAdcData::bias_type FrAdcData::
    GetBias( ) const
    {
      return m_data.bias;
    }

    inline FrAdcData::slope_type FrAdcData::
    GetSlope( ) const 
    {
      return m_data.slope;
    }

    inline STRING FrAdcData::
    GetUnits( ) const
    {
      return m_data.units;
    }

    inline FrAdcData::sampleRate_type FrAdcData::
    GetSampleRate( ) const
    {
      return m_data.sampleRate;
    }

    inline FrAdcData::timeOffset_type FrAdcData::
    GetTimeOffset( ) const
    {
      return m_data.timeOffset;
    }

    inline FrAdcData::fShift_type FrAdcData::
    GetFShift( ) const
    {
      return m_data.fShift;
    }

    inline FrAdcData::phase_type FrAdcData::
    GetPhase( ) const
    {
      return m_data.phase;
    }

    inline FrAdcData::dataValid_type FrAdcData::
    GetDataValid( ) const
    {
      return m_data.dataValid;
    }

    inline const FrAdcData::aux_type& FrAdcData::
    RefAux( ) const
    {
      return m_data.aux;
    }

    inline FrAdcData::aux_type& FrAdcData::
    RefAux( )
    {
      return m_data.aux;
    }

    inline const FrAdcData::data_type& FrAdcData::
    RefData( ) const
    {
      return m_data.data;
    }

    inline FrAdcData::data_type& FrAdcData::
    RefData( )
    {
      return m_data.data;
    }

    inline void FrAdcData::
    SetDataValid( dataValid_type flag )
    {
      m_data.dataValid = flag;
    }

    inline void FrAdcData::
    SetBias( bias_type bias )
    {
      m_data.bias = bias;
    }

    inline void FrAdcData::
    SetSampleRate( sampleRate_type SampleRate )
    {
      m_data.sampleRate = SampleRate;
#if WORKING
      //-----------------------------------------------------------------
      /// \todo
      /// Modifications to the sample rate need also be reflected in the
      /// corrisponding FrVect structures.
      //-----------------------------------------------------------------
      for ( data_type::iterator
	      cur = RefData( ).begin( ),
	      last = RefData( ).end( );
	    cur != last;
	    ++cur )
      {
	cur->
      }
#endif /* WORKING */
    }

    inline void FrAdcData::
    SetSlope( slope_type slope )
    {
      m_data.slope = slope;
    }

    inline void FrAdcData::
    SetTimeOffset( timeOffset_type time )
    {
      m_data.timeOffset = time;
    }

    inline bool FrAdcData::data_definition_type::
    operator!=( const data_definition_type& RHS ) const
    {
      return ! ( *this == RHS );
    }

    inline void FrAdcData::
    AppendComment( const comment_type& comment )
    {
      FrameCPP::Common::AppendComment( m_data.comment, comment );
    } 

    inline bool FrAdcData::
    operator==( const FrAdcData& RHS ) const
    {
      return ( this->m_data == RHS.m_data );
    }

    inline bool FrAdcData::
    operator!=( const FrAdcData& RHS ) const
    {
      return ! ( *this == RHS );
    }
  } // namespace - Version_8
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_8_FrAdcData_HH */
